# Source-based routing in Golang


Consider this routing table:
```
10.11.70.0/23 dev data0 proto kernel scope link src 10.11.70.42
10.11.70.0/23 dev data1 proto kernel scope link src 10.11.71.42
192.168.123.0/24 dev internal0 proto kernel scope link src 192.168.123.42
192.168.123.0/24 dev internal1 proto kernel scope link src 192.168.123.142
```

Simple `net.Dial` to either `10.11.70.42` or `10.11.71.42` will match the first subnet and be routed via data0.
This problems is usually solved by bonds.
But sometimes you need to invent a bicycle.

## Usage

```golang
import (
    "context"
    "net"
    "net/netip"

    "git.frostfs.info/TrueCloudLab/multinet"
)

d, err := multinet.NewDialer(Config{
    Subnets:  []Subnet{
        {
            Prefix: netip.MustParsePrefix("10.11.70.0/23"),
            SourceIPs: []netip.Addr{
                netip.MustParseAddr("10.11.70.42"),
                netip.MustParseAddr("10.11.71.42"),
            },
        },
        {
            Prefix: netip.MustParsePrefix("192.168.123.0/24"),
            SourceIPs: []netip.Addr{
                netip.MustParseAddr("192.168.123.42"),
                netip.MustParseAddr("192.168.123.142"),
            },
        },
    },
    Balancer: multinet.BalancerTypeRoundRobin,
})
if err != nil {
    // handle error
}

conn, err := d.DialContext(ctx, "tcp", "10.11.70.42")
if err != nil {
    // handle error
}
// do stuff
```

## License and copyright

Copyright 2023-2024 FrostFS contributors

```
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
```
